import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:flutter_wni/flutter_wni.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:webview_flutter/webview_flutter.dart';
import 'package:permission_handler/permission_handler.dart';

class WebPage extends StatefulWidget {
  final String url;

  const WebPage({Key? key, required this.url}) : super(key: key);

  @override
  State<WebPage> createState() => _WebPageState();
}

class _WebPageState extends State<WebPage> {
  late WNInterface _interface;

  @override
  Widget build(BuildContext context) {
    var that = this;
    return WillPopScope(
      onWillPop: () => goBack(context),
      child: Stack(
        children: [
          WebView(
            debuggingEnabled: true,
            initialUrl: widget.url,
            userAgent: WNInterface.userAgent(),
            javascriptMode: JavascriptMode.unrestricted,
            javascriptChannels: {
              WNInterface.getJavascriptChannel(
                  (payload) => _interface.execute(payload))
            },
            onWebViewCreated: (WebViewController controller) async {
              _interface = WNInterfaceCustom(controller: controller, context: context, parent: that);
              _interface.controller.clearCache();
            },
          ),
        ],
      ),
    );
  }

  // LifeCycle Methods
  void onContentLoaded() async {
    onReady();
  }

  void onReady() async {
    _interface.call("WNInterface.responder.onInitPage", jsonEncode({"status": "SUCCESS"}));
  }

  void onAppear() async {}

  void onDisappear() async {}

  void onLink() async {}

  void onCustom(userInfo) async {
    _interface.call("WNInterface.responder.onRemoteNotification", jsonEncode(userInfo));
  }

  void onBackPressed() async {
    _interface.call("WNInterface.responder.onBackPage", jsonEncode({}));
  }

  void onRemoteNotification(userInfo) async {
    _interface.call("WNInterface.responder.onRemoteNotification", jsonEncode(userInfo));
  }

  Future<bool> goBack(BuildContext context) async {
    

    return Future.value(false);
  }
}

class WNInterfaceCustom extends WNInterface {
  late dynamic _parent;
  get parent => _parent;

  WNInterfaceCustom({controller, context, required dynamic parent})
      : super(controller: controller, context: context) {
        _parent = parent;

    registerMethods({
      "wnWebContentLoaded": wnWebContentLoaded,
      "wnAppChangeStatusBar": wnAppChangeStatusBar,
      "wnAppInfo": wnAppInfo,
      "wnGetSafeArea": wnGetSafeArea,
      "wnAllowAccess": wnAllowAccess,
      "wnGetStorage": wnGetStorage,
      "wnSetStorage": wnSetStorage,
    });
  }

  // WNInterface
  void wnWebContentLoaded(data) async {
    parent.onContentLoaded();
  }

  // Log
  void wnLog(data) async {
    // 
  }

  // App
  void wnAppChangeStatusBar(data) async {
    // 
  }

  void wnAppInfo(data) async {
    final String callback = data["callback"] as String;
    call(callback, jsonEncode({
      "status": "SUCCESS",
      "deviceLocale": "ko",
      "deviceId": "229df161194d0f4b6385e61895022688d6"
    }));
  }

  void wnGetSafeArea(data) async {
    final String callback = data["callback"] as String;
    final EdgeInsets safeArea = MediaQuery.of(context).padding;
    final Map<String, dynamic> result = <String, dynamic>{};
    result["status"] = "SUCCESS";
    result["top"] = safeArea.top;
    result["left"] = safeArea.left;
    result["right"] = safeArea.right;
    result["bottom"] = safeArea.bottom;

    call(callback, jsonEncode(result));
  }

  void wnAllowAccess(data) async {
    final String type = data["type"] as String;
    final String callback = data["callback"] as String;

    PermissionStatus? permissionStatus;

    if (type == "camera") {
      permissionStatus = await Permission.camera.request();
    } else if (type == "location") {
      permissionStatus = await Permission.location.request();
    } else if (type == "microphone") {
      permissionStatus = await Permission.microphone.request();
    } else if (type == "calendar") {
      permissionStatus = await Permission.calendar.request();
    } else if (type == "phone") {
      permissionStatus = await Permission.phone.request();
    } else if (type == "contacts") {
      permissionStatus = await Permission.contacts.request();
    } else if (type == "photos") {
      permissionStatus = await Permission.photos.request();
    } else if (type == "notification") {
      permissionStatus = await Permission.notification.request();
    }

    String status = "UNKNOWN";
    if (permissionStatus != null) {
      switch (permissionStatus) {
        case PermissionStatus.denied:
        case PermissionStatus.permanentlyDenied:
          status = "DENIED";
          break;

        case PermissionStatus.granted:
          status = "GRANTED";
          break;

        case PermissionStatus.limited:
          status = "LIMITED";
          break;

        case PermissionStatus.restricted:
          status = "RESTRICTED";
          break;

        default:
          break;
      }
    }

    // ignore: avoid_print
    print("status: " + status);

    call(callback, jsonEncode({ "status": status, }));
  }

  void wnGetStorage(data) async {
    final String callback = data["callback"] as String;
    final String key = data["key"] as String;
    final prefs = await SharedPreferences.getInstance();
    final String? value = prefs.getString(key);

    call(callback, jsonEncode({"status": "SUCCESS", "key": key, "value": value ?? ""}));
  }

  void wnSetStorage(data) async {
    final String callback = data["callback"] as String;
    final String key = data["key"] as String;
    final String value = data["value"] as String;
    final prefs = await SharedPreferences.getInstance();
    prefs.setString(key, value);

    call(callback, jsonEncode({"status": "SUCCESS", "key": key, "value": value}));
  }

  // ---- 정리 필요

  // void wnAppleLogin(data) async {
  //   final String callback = data["callback"] as String;

  //   var apple = new appleLogin();
  //   var value = await apple.signInWithApple();
  //   if (value != null) {
  //     String? email = value as String;
  //     call(callback, jsonEncode({"email": email}));
  //   } else {
  //     print('failed');
  //   }
  // }

  // //카카오 맵 딥링크 연결
  // void wnKakaoMap(data) async {
  //   final String urlObject = data["Url"] as String;
  //   await launch(urlObject);
  // }

  // void wnKaKaoLogin(data, email) async {
  //   final String callback = data["callback"] as String;
  //   call(callback, jsonEncode({"email": email}));
  // }

  /*
  void wnStartScanning(data) async {
    final String callback = data["callback"] as String;

    QRScanner.start(
      context: context,
      onScan: (code) {
        // ignore: avoid_print
        print("code " + code);

        call(callback, jsonEncode({"status": "SUCCESS", "code": code}));
      },
    );
  }

  void wnStopScanning(data) async {
    final String callback = data["callback"] as String;

    QRScanner.stop(context: context);

    call(
        callback,
        jsonEncode({
          "status": "SUCCESS",
        }));
  }
  */
}
